package dev.micle.totemofreviving.network;

import dev.micle.totemofreviving.TotemOfReviving;
import dev.micle.totemofreviving.item.totem.TotemItem;
import net.minecraft.ChatFormatting;
import net.minecraft.network.FriendlyByteBuf;
import net.minecraft.network.chat.TextComponent;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.server.players.PlayerList;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.item.ItemStack;
import net.minecraftforge.fmllegacy.network.NetworkEvent;

import java.util.function.Supplier;

public class ChangeTargetPacket {
	private final InteractionHand hand;
	
	public ChangeTargetPacket(final InteractionHand hand) {
		this.hand = hand;
	}
	
	public static void encode(final ChangeTargetPacket packet, final FriendlyByteBuf buffer) {
		Network.writeVersionInfo(buffer, false);
		buffer.writeEnum(packet.hand);
	}
	
	public static ChangeTargetPacket decode(final FriendlyByteBuf buffer) {
		Network.checkVersion(buffer);
		return new ChangeTargetPacket(buffer.readEnum(InteractionHand.class));
	}
	
	public static void handle(final ChangeTargetPacket packet, final Supplier<NetworkEvent.Context> contextSupplier) {
		final NetworkEvent.Context context = contextSupplier.get();
		context.enqueueWork(() -> {
			ServerPlayer sender = context.getSender();
			if (sender == null) { return; }
			
			ItemStack totemStack = sender.getItemInHand(packet.hand);
			PlayerList playerList = TotemOfReviving.PROXY.getServer().getPlayerList();
			int targetIndex = TotemItem.getTargetIndex(totemStack) + 1;
			
			if (targetIndex > playerList.getPlayerCount()-1) { targetIndex = 0; }
			ServerPlayer target = playerList.getPlayers().get(targetIndex);
			
			TotemItem.setTargetIndex(totemStack, targetIndex);
			TotemItem.setTargetUUID(totemStack, target.getUUID());
			TotemItem.setTargetName(totemStack, target.getDisplayName().getString());
			TotemItem.setTargetDeaths(totemStack, target);
			
			sender.sendMessage(new TextComponent(ChatFormatting.WHITE + "Now targetting " + ChatFormatting.GRAY + target.getDisplayName().getString() + "."), sender.getUUID());
		});
		context.setPacketHandled(true);
	}
}
